@@ -11,7 +11,7 @@ from scipy import stats as ss
 from sklearn import metrics
 from itertools import product
 
-from .misc import LoggingMixin, get_one_hot
+from .misc import *
 
 
 class Anneal:
@@ -270,6 +270,35 @@ class Metrics(LoggingMixin):
         return thresholds[np.argmax(metric)]
 
 
+class Estimator:
+    def __init__(self, metric: str, **kwargs):
+        self._metric = Metrics(metric, **kwargs)
+
+    def estimate(self,
+                 x: np.ndarray,
+                 y: np.ndarray,
+                 methods: Dict[str, Union[None, Callable]]):
+        scores = {
+            name: None if method is None else self._metric.score(y, method(x))
+            for name, method in methods.items()
+        }
+        msg_list = []
+        best_idx, best_score = -1, -math.inf
+        for i, name in enumerate(sorted(scores)):
+            score = scores[name]
+            if score is None:
+                continue
+            msg_list.append(f"|  {name:>20s}  |  {self._metric.type:^8s}  |  {score:8.6f}  |")
+            new_score = score * self._metric.sign
+            if new_score > best_score:
+                best_idx, best_score = i, new_score
+        msg_list[best_idx] += "  <-  "
+        width = max(map(len, msg_list))
+        msg_list.insert(0, "=" * width)
+        msg_list.append("-" * width)
+        print("\n".join(msg_list))
+
+
 class ScalarEMA:
     """
     Util class to record Exponential Moving Average (EMA) for scalar value
@@ -457,5 +486,83 @@ class Visualizer:
                 save_name = self._get_save_name(save_name)
             plt.savefig(os.path.join(self.export_folder, f"{save_name}.png"))
 
+    @staticmethod
+    def visualize1d(method: Callable,
+                    x: np.ndarray,
+                    y: np.ndarray = None,
+                    *,
+                    title: str = None,
+                    num_samples: int = 100,
+                    expand_ratio: float = 0.25,
+                    return_canvas: bool = False) -> Union[None, np.ndarray]:
+        if x.shape[1] != 1:
+            raise ValueError("visualize1d only supports 1-dimensional features")
+        plt.figure()
+        plt.title(title)
+        if y is not None:
+            plt.scatter(x, y, c="g", s=20)
+        x_min, x_max = x.min(), x.max()
+        expand = expand_ratio * (x_max - x_min)
+        x0 = np.linspace(x_min - expand, x_max + expand, num_samples).reshape([-1, 1])
+        plt.plot(x0, method(x0).ravel())
+        return show_or_return(return_canvas)
 
-__all__ = ["Anneal", "Metrics", "ScalarEMA", "Grid", "Visualizer"]
+    @staticmethod
+    def visualize2d(method,
+                    x: np.ndarray,
+                    y: np.ndarray = None,
+                    *,
+                    title: str = None,
+                    dense: int = 200,
+                    padding: float = 0.1,
+                    return_canvas: bool = False,
+                    draw_background: bool = True,
+                    extra_scatters: np.ndarray = None,
+                    emphasize_indices: np.ndarray = None) -> Union[None, np.ndarray]:
+        axis = x.T
+        if axis.shape[0] != 2:
+            raise ValueError("visualize2d only supports 2-dimensional features")
+        nx, ny, padding = dense, dense, padding
+        x_min, x_max = np.min(axis[0]), np.max(axis[0])  # type: float
+        y_min, y_max = np.min(axis[1]), np.max(axis[1])  # type: float
+        x_padding = max(abs(x_min), abs(x_max)) * padding
+        y_padding = max(abs(y_min), abs(y_max)) * padding
+        x_min -= x_padding
+        x_max += x_padding
+        y_min -= y_padding
+        y_max += y_padding
+
+        def get_base(_nx, _ny):
+            _xf = np.linspace(x_min, x_max, _nx)
+            _yf = np.linspace(y_min, y_max, _ny)
+            n_xf, n_yf = np.meshgrid(_xf, _yf)
+            return _xf, _yf, np.c_[n_xf.ravel(), n_yf.ravel()]
+
+        xf, yf, base_matrix = get_base(nx, ny)
+        z = method(base_matrix).reshape((nx, ny))
+
+        labels = y.ravel()
+        num_labels = y.max().item() + 1
+        colors = plt.cm.rainbow([i / num_labels for i in range(num_labels)])[labels]
+
+        plt.figure()
+        plt.title(title)
+        if draw_background:
+            xy_xf, xy_yf = np.meshgrid(xf, yf, sparse=True)
+            plt.pcolormesh(xy_xf, xy_yf, z, cmap=plt.cm.Pastel1)
+        else:
+            plt.contour(xf, yf, z, c='k-', levels=[0])
+        plt.scatter(axis[0], axis[1], c=colors)
+
+        if emphasize_indices is not None:
+            indices = np.array([False] * len(axis[0]))
+            indices[np.asarray(emphasize_indices)] = True
+            plt.scatter(axis[0][indices], axis[1][indices], s=80,
+                        facecolors="None", zorder=10)
+        if extra_scatters is not None:
+            plt.scatter(*np.asarray(extra_scatters).T, s=80, zorder=25, facecolors="red")
+
+        return show_or_return(return_canvas)
+
+
+__all__ = ["Anneal", "Metrics", "Estimator", "ScalarEMA", "Grid", "Visualizer"]
